@@ -26,16 +26,21 @@ from genomepy.utils import (
     read_url,
     safe,
     check_url,
+    retry,
     is_number,
     mkdir_p,
+    get_genomes_dir,
 )
 from genomepy.__about__ import __version__
 
-# store the output of slow commands (marked with @cached) for fast reuse
+# Store the output of slow commands (marked with @cache and @goldfish_cache) for fast reuse.
+# Bucketcache creates a new pickle for each function + set of unique variables,
+# to avoid storing self.genomes, ignore=["self"] or use @staticmethod.
 my_cache_dir = os.path.join(user_cache_dir("genomepy"), __version__)
 if not os.path.exists(my_cache_dir):
     os.makedirs(my_cache_dir)
-cached = Bucket(my_cache_dir, days=7)
+cache = Bucket(my_cache_dir, days=7)
+goldfish_cache = Bucket(my_cache_dir, minutes=10)
 
 config = norns.config("genomepy", default="cfg/default.yaml")
 
@@ -94,6 +99,12 @@ class ProviderBase(object):
 
         return decorator
 
+    @goldfish_cache(ignore=["self", "max_tries"])
+    def provider_status(self, url, max_tries=1):
+        """check if provider is online (stores results for 10 minutes)"""
+        if not check_url(url, max_tries):
+            raise ConnectionError(f"{self.name} appears to be offline.\n")
+
     @classmethod
     def list_providers(cls):
         """List available providers."""
@@ -173,9 +184,10 @@ class ProviderBase(object):
     def download_genome(
         self,
         name,
-        genomes_dir,
+        genomes_dir=None,
         localname=None,
         mask="soft",
+        keep_alt=False,
         regex=None,
         invert_match=False,
         bgzip=None,
@@ -189,7 +201,7 @@ class ProviderBase(object):
         name : str
             Genome / species name
 
-        genomes_dir : str
+        genomes_dir : str , optional
             Directory to install genome
 
         localname : str , optional
@@ -198,6 +210,9 @@ class ProviderBase(object):
         mask: str , optional
             Masking, soft, hard or none (all other strings)
 
+        keep_alt : bool , optional
+            Set to true to keep these alternative regions.
+
         regex : str , optional
             Regular expression to select specific chromosome / scaffold names.
 
@@ -208,20 +223,20 @@ class ProviderBase(object):
             If set to True the genome FASTA file will be compressed using bgzip.
             If not specified, the setting from the configuration file will be used.
         """
+        name = safe(name)
         self.check_name(name)
 
         link = self.get_genome_download_link(name, mask=mask, **kwargs)
 
-        original_name = name
-        name = safe(name)
         localname = get_localname(name, localname)
-
-        genomes_dir = os.path.expanduser(genomes_dir)
+        genomes_dir = get_genomes_dir(genomes_dir, check_exist=False)
         out_dir = os.path.join(genomes_dir, localname)
         if not os.path.exists(out_dir):
             mkdir_p(out_dir)
 
-        sys.stderr.write(f"Downloading genome from {link}...\n")
+        sys.stderr.write(
+            f"Downloading genome from {self.name}.\nTarget URL: {link}...\n"
+        )
 
         # download to tmp dir. Move genome on completion.
         # tmp dir is in genome_dir to prevent moving the genome between disks
@@ -252,22 +267,29 @@ class ProviderBase(object):
                 if ret != 0:
                     raise Exception(f"Error gunzipping genome {fname}")
 
+            def regex_filer(_fname, _regex, _v):
+                os.rename(_fname, _fname + "_to_regex")
+                infa = _fname + "_to_regex"
+                outfa = _fname
+                filter_fasta(infa, outfa, regex=_regex, v=_v, force=True)
+
+                return [k for k in Fasta(infa).keys() if k not in Fasta(outfa).keys()]
+
+            not_included = []
+            # remove alternative regions
+            if not keep_alt:
+                not_included.extend(regex_filer(fname, "alt", True))
+
+            # keep/remove user defined regions
+            if regex:
+                not_included.extend(regex_filer(fname, regex, invert_match))
+
             # process genome (e.g. masking)
             if hasattr(self, "_post_process_download"):
                 self._post_process_download(
                     name=name, localname=localname, out_dir=tmp_dir, mask=mask
                 )
 
-            if regex:
-                os.rename(fname, fname + "_to_regex")
-                infa = fname + "_to_regex"
-                outfa = fname
-                filter_fasta(infa, outfa, regex=regex, v=invert_match, force=True)
-
-                not_included = [
-                    k for k in Fasta(infa).keys() if k not in Fasta(outfa).keys()
-                ]
-
             # bgzip genome if requested
             if bgzip or config.get("bgzip"):
                 ret = sp.check_call(["bgzip", "-f", fname])
@@ -290,7 +312,7 @@ class ProviderBase(object):
         metadata = {
             "name": localname,
             "provider": self.name,
-            "original name": original_name,
+            "original name": name,
             "original filename": os.path.split(link)[-1],
             "assembly_accession": self.assembly_accession(self.genomes.get(name)),
             "tax_id": self.genome_taxid(self.genomes.get(name)),
@@ -392,7 +414,9 @@ class ProviderBase(object):
             )
             return
 
-        sys.stderr.write(f"\nDownloading annotation from {link}...\n")
+        sys.stderr.write(
+            f"Downloading annotation from {self.name}.\nTarget URL: {link}...\n"
+        )
         try:
             self.download_and_generate_annotation(genomes_dir, link, localname)
         except Exception:
@@ -411,7 +435,7 @@ class ProviderBase(object):
         metadata["annotation url"] = link
         write_readme(readme, metadata, lines)
 
-    def download_annotation(self, name, genomes_dir, localname=None, **kwargs):
+    def download_annotation(self, name, genomes_dir=None, localname=None, **kwargs):
         """
         Download annotation file to to a specific directory
 
@@ -420,7 +444,7 @@ class ProviderBase(object):
         name : str
             Genome / species name
 
-        genomes_dir : str
+        genomes_dir : str , optional
             Directory to install annotation
 
         localname : str , optional
@@ -430,6 +454,8 @@ class ProviderBase(object):
 
         link = self.get_annotation_download_link(name, **kwargs)
 
+        localname = get_localname(name, localname)
+        genomes_dir = get_genomes_dir(genomes_dir, check_exist=False)
         self.attempt_and_report(name, localname, link, genomes_dir)
 
     def _search_taxids(self, genome, term):
@@ -444,6 +470,29 @@ class ProviderBase(object):
             if term in safe(genome[field].lower()):
                 return True
 
+    def _search_accessions(self, term):
+        """
+        Search for specific assembly accession.
+
+        Parameters
+        ----------
+        term : str
+            Assembly accession, GCA_....
+        Yields
+        ------
+        tuples with name and metadata
+        """
+        # NCBI provides a consistent assembly accession. This can be used to
+        # retrieve the species, and then search for that.
+        p = ProviderBase.create("NCBI")
+        species = [row[2] for row in p.search(term)]
+        if len(species) == 0:
+            raise ValueError(f"No genome found with accession {term}")
+        species = species[0]
+        for row in self.search(species):
+            if row[1] == term:
+                yield row
+
     def search(self, term):
         """
         Search for term in genome names, descriptions and taxonomy ID.
@@ -460,20 +509,23 @@ class ProviderBase(object):
         ------
         tuples with name and metadata
         """
-        term = str(term)
         genomes = self.genomes
-        if safe(term) in genomes:
-            yield self._genome_info_tuple(term)
+        term = safe(str(term))
+        if term.startswith("GCA_") and self.name != "NCBI":
+            for row in self._search_accessions(term):
+                yield (row)
 
         elif is_number(term):
             for name in genomes:
-                if self._search_taxids(genome=genomes[name], term=term):
+                if self._search_taxids(genomes[name], term):
                     yield self._genome_info_tuple(name)
 
         else:
-            term = safe(term).lower()
+            term = term.lower()
             for name in genomes:
-                if self._search_descriptions(genome=genomes[name], term=term):
+                if term in safe(name).lower() or self._search_descriptions(
+                    genomes[name], term
+                ):
                     yield self._genome_info_tuple(name)
 
 
@@ -489,7 +541,7 @@ class EnsemblProvider(ProviderBase):
     The bacteria division is not yet supported.
     """
 
-    rest_url = "http://rest.ensembl.org/"
+    rest_url = "https://rest.ensembl.org/"
     provider_specific_install_options = {
         "toplevel": {
             "long": "toplevel",
@@ -505,11 +557,10 @@ class EnsemblProvider(ProviderBase):
     }
 
     def __init__(self):
-        # Necessary for bucketcache, otherwise methods with identical names
-        # from different classes will use the same cache :-O!
         self.name = "Ensembl"
+        self.provider_status(self.rest_url + "info/ping?", max_tries=2)
         # Populate on init, so that methods can be cached
-        self.genomes = self._get_genomes()
+        self.genomes = self._get_genomes(self.rest_url)
         self.accession_fields = ["assembly_accession"]
         self.taxid_fields = ["taxonomy_id"]
         self.description_fields = [
@@ -518,34 +569,31 @@ class EnsemblProvider(ProviderBase):
             "url_name",
             "display_name",
         ]
-        self.version = None
 
-    @cached(method=True)
-    def _request_json(self, ext):
+    @staticmethod
+    def _request_json(rest_url, ext):
         """Make a REST request and return as json."""
-        if self.rest_url.endswith("/") and ext.startswith("/"):
+        if rest_url.endswith("/") and ext.startswith("/"):
             ext = ext[1:]
 
-        r = requests.get(
-            self.rest_url + ext, headers={"Content-Type": "application/json"}
-        )
+        r = requests.get(rest_url + ext, headers={"Content-Type": "application/json"})
 
         if not r.ok:
             r.raise_for_status()
 
         return r.json()
 
-    @cached(method=True)
-    def _get_genomes(self):
+    @cache(ignore=["self"])
+    def _get_genomes(self, rest_url):
         sys.stderr.write("Downloading assembly summaries from Ensembl\n")
 
         genomes = {}
-        divisions = self._request_json("info/divisions?")
+        divisions = retry(self._request_json, 3, rest_url, "info/divisions?")
         for division in divisions:
             if division == "EnsemblBacteria":
                 continue
-            division_genomes = self._request_json(
-                "info/genomes/division/{}?".format(division)
+            division_genomes = retry(
+                self._request_json, 3, rest_url, f"info/genomes/division/{division}?"
             )
             for genome in division_genomes:
                 genomes[safe(genome["assembly_name"])] = genome
@@ -566,17 +614,13 @@ class EnsemblProvider(ProviderBase):
             genome.get("genebuild", "na"),
         )
 
-    def get_version(self, ftp_site):
-        """Retrieve current version from Ensembl FTP.
-        """
-        with urlopen(ftp_site + "/current_README") as response:
-            p = re.compile(r"Ensembl (Genomes|Release) (\d+)")
-            m = p.search(response.read().decode())
-        if m:
-            version = m.group(2)
-            sys.stderr.write("Using version {}\n".format(version))
-            self.version = version
-            return version
+    @goldfish_cache(ignore=["self", "rest_url"])
+    def get_version(self, rest_url, vertebrates=False):
+        """Retrieve current version from Ensembl FTP."""
+        ext = "/info/data/?" if vertebrates else "/info/eg_version?"
+        ret = retry(self._request_json, 3, rest_url, ext)
+        releases = ret["releases"] if vertebrates else [ret["version"]]
+        return str(max(releases))
 
     def get_genome_download_link(self, name, mask="soft", **kwargs):
         """
@@ -604,14 +648,12 @@ class EnsemblProvider(ProviderBase):
 
         ftp_site = "ftp://ftp.ensemblgenomes.org/pub"
         if division == "vertebrates":
-            ftp_site = "http://ftp.ensembl.org/pub"
+            ftp_site = "ftp://ftp.ensembl.org/pub"
 
         # Ensembl release version
-        version = self.version
-        if kwargs.get("version"):
-            version = kwargs.get("version")
-        elif not version:
-            version = self.get_version(ftp_site)
+        version = kwargs.get("version")
+        if version is None:
+            version = self.get_version(self.rest_url, division == "vertebrates")
 
         # division dependent url format
         ftp_dir = "{}/release-{}/fasta/{}/dna".format(
@@ -639,10 +681,10 @@ class EnsemblProvider(ProviderBase):
         # try to get the (much smaller) primary assembly,
         # unless specified otherwise
         link = get_url("primary_assembly")
-        if kwargs.get("toplevel") or not check_url(link):
+        if kwargs.get("toplevel") or not check_url(link, 2):
             link = get_url()
 
-        if check_url(link):
+        if check_url(link, 2):
             return link
 
         raise GenomeDownloadError(
@@ -670,14 +712,12 @@ class EnsemblProvider(ProviderBase):
 
         ftp_site = "ftp://ftp.ensemblgenomes.org/pub"
         if division == "vertebrates":
-            ftp_site = "http://ftp.ensembl.org/pub"
+            ftp_site = "ftp://ftp.ensembl.org/pub"
 
         # Ensembl release version
-        version = self.version
-        if kwargs.get("version"):
-            version = kwargs.get("version")
-        elif not version:
-            version = self.get_version(ftp_site)
+        version = kwargs.get("version")
+        if version is None:
+            version = self.get_version(self.rest_url, division == "vertebrates")
 
         if division != "vertebrates":
             ftp_site += f"/{division}"
@@ -693,7 +733,7 @@ class EnsemblProvider(ProviderBase):
             version,
         )
 
-        if check_url(link):
+        if check_url(link, 2):
             return link
 
 
@@ -720,32 +760,32 @@ class UcscProvider(ProviderBase):
     }
 
     def __init__(self):
-        # Necessary for bucketcache, otherwise methods with identical names
-        # from different classes will use the same cache :-O!
         self.name = "UCSC"
+        self.provider_status(self.base_url)
         # Populate on init, so that methods can be cached
-        self.genomes = self._get_genomes()
+        self.genomes = self._get_genomes(self.rest_url)
         self.accession_fields = []
         self.taxid_fields = ["taxId"]
         self.description_fields = ["description", "scientificName"]
 
-    @cached(method=True)
-    def _get_genomes(self):
+    @staticmethod
+    @cache
+    def _get_genomes(rest_url):
         sys.stderr.write("Downloading assembly summaries from UCSC\n")
 
-        r = requests.get(self.rest_url, headers={"Content-Type": "application/json"})
+        r = requests.get(rest_url, headers={"Content-Type": "application/json"})
         if not r.ok:
             r.raise_for_status()
         ucsc_json = r.json()
         genomes = ucsc_json["ucscGenomes"]
         return genomes
 
-    # staticmethod, but the decorators cannot be combined
-    @cached(method=True)
-    def assembly_accession(self, genome):
+    @staticmethod
+    @cache
+    def assembly_accession(genome):
         """Return the assembly accession (GCA_*) for a genome.
 
-        UCSC does not server the assembly accession through the REST API.
+        UCSC does not serve the assembly accession through the REST API.
         Therefore, the readme.html is scanned for a GCA assembly id. If it is
         not found, the linked NCBI assembly page will be checked. Especially
         for older genome builds, the GCA will not be present, in which case
@@ -832,7 +872,7 @@ class UcscProvider(ProviderBase):
         for genome_url in urls:
             link = genome_url.format(name)
 
-            if check_url(link):
+            if check_url(link, 2):
                 return link
 
         raise GenomeDownloadError(
@@ -898,7 +938,7 @@ class UcscProvider(ProviderBase):
         }
 
         # download gtf format if possible, txt format if not
-        gtfs_exists = check_url(gtf_url)
+        gtfs_exists = check_url(gtf_url, 2)
         base_url = gtf_url + name + "." if gtfs_exists else txt_url
         base_ext = ".gtf.gz" if gtfs_exists else ".txt.gz"
 
@@ -906,7 +946,7 @@ class UcscProvider(ProviderBase):
         file = kwargs.get("ucsc_annotation_type")
         if file:
             link = base_url + annot_files[file.lower()] + base_ext
-            if check_url(link):
+            if check_url(link, 2):
                 return link
             sys.stderr.write(
                 f"Specified annotation type ({file}) not found for {name}.\n"
@@ -916,7 +956,7 @@ class UcscProvider(ProviderBase):
             # download first available annotation type found
             for file in annot_files.values():
                 link = base_url + file + base_ext
-                if check_url(link):
+                if check_url(link, 2):
                     return link
 
 
@@ -932,17 +972,23 @@ class NcbiProvider(ProviderBase):
     provider_specific_install_options = {}
 
     def __init__(self):
-        # Necessary for bucketcache, otherwise methods with identical names
-        # from different classes will use the same cache :-O!
         self.name = "NCBI"
+        self.provider_status(self.assembly_url)
         # Populate on init, so that methods can be cached
-        self.genomes = self._get_genomes()
+        self.genomes = self._get_genomes(self.assembly_url)
         self.accession_fields = ["assembly_accession", "gbrs_paired_asm"]
         self.taxid_fields = ["species_taxid", "taxid"]
-        self.description_fields = ["submitter", "organism_name"]
+        self.description_fields = [
+            "submitter",
+            "organism_name",
+            "assembly_accession",
+            "gbrs_paired_asm",
+            "paired_asm_comp",
+        ]
 
-    @cached(method=True)
-    def _get_genomes(self):
+    @staticmethod
+    @cache
+    def _get_genomes(assembly_url):
         """Parse genomes from assembly summary txt files."""
         sys.stderr.write(
             "Downloading assembly summaries from NCBI, this will take a while...\n"
@@ -957,7 +1003,7 @@ class NcbiProvider(ProviderBase):
         ]
         for fname in names:
             urlcleanup()
-            with urlopen(os.path.join(self.assembly_url, fname)) as response:
+            with urlopen(os.path.join(assembly_url, fname)) as response:
                 lines = response.read().decode("utf-8").splitlines()
             header = lines[1].strip("# ").split("\t")
             for line in lines[2:]:
@@ -1005,7 +1051,7 @@ class NcbiProvider(ProviderBase):
         link = link.replace("ftp://", "https://")
         link += "/" + link.split("/")[-1] + "_genomic.fna.gz"
 
-        if check_url(link):
+        if check_url(link, 2):
             return link
 
         raise GenomeDownloadError(
@@ -1096,7 +1142,7 @@ class NcbiProvider(ProviderBase):
         link = link.replace("ftp://", "https://")
         link += "/" + link.split("/")[-1] + "_genomic.gff.gz"
 
-        if check_url(link):
+        if check_url(link, 2):
             return link
 
 
@@ -1196,7 +1242,7 @@ class UrlProvider(ProviderBase):
         if check_url(link):
             return link
 
-    def download_annotation(self, url, genomes_dir, localname=None, **kwargs):
+    def download_annotation(self, url, genomes_dir=None, localname=None, **kwargs):
         """
         Attempts to download a gtf or gff3 file from the same location as the genome url
 
@@ -1219,6 +1265,7 @@ class UrlProvider(ProviderBase):
         """
         name = get_localname(url)
         localname = get_localname(name, localname)
+        genomes_dir = get_genomes_dir(genomes_dir, check_exist=False)
 
         if kwargs.get("to_annotation"):
             link = self.get_annotation_download_link(None, **kwargs)
