@@ -13,6 +13,7 @@ import time
 from glob import glob
 from norns import exceptions
 from pyfaidx import Fasta
+from socket import timeout
 from tempfile import TemporaryDirectory
 
 config = norns.config("genomepy", default="cfg/default.yaml")
@@ -80,7 +81,7 @@ def write_readme(readme, metadata, lines):
 
 
 def generate_gap_bed(fname, outname):
-    """ Generate a BED file with gap locations.
+    """Generate a BED file with gap locations.
 
     Parameters
     ----------
@@ -98,7 +99,7 @@ def generate_gap_bed(fname, outname):
 
 
 def generate_fa_sizes(fname, outname):
-    """ Generate a fa.sizes file.
+    """Generate a fa.sizes file.
 
     Parameters
     ----------
@@ -176,6 +177,11 @@ def mkdir_p(path):
     os.makedirs(path, exist_ok=True)
 
 
+def rm_rf(path):
+    """ 'rm -rf' in Python """
+    shutil.rmtree(path, ignore_errors=True)
+
+
 def cmd_ok(cmd):
     """Returns True if cmd can be run."""
     try:
@@ -210,17 +216,15 @@ def glob_ext_files(dirname, ext="fa"):
     dirname: str
         Directory name.
 
-    ext: str
+    ext: str , optional
         Filename extension (default: fa).
 
     Returns
     -------
         File names.
     """
-    fnames = glob(os.path.join(dirname, "*." + ext + "*"))
-    return [
-        fname for fname in fnames if fname.endswith(ext) or fname.endswith(ext + ".gz")
-    ]
+    fnames = glob(os.path.join(dirname, f"*.{ext}*"))
+    return [f for f in fnames if f.endswith((ext, f"{ext}.gz"))]
 
 
 def get_genomes_dir(genomes_dir=None, check_exist=True):
@@ -335,15 +339,40 @@ def is_number(term):
         return True
 
 
-def check_url(url):
-    """Check if URL works. Returns bool"""
+def try_except_pass(errors, func, *args):
+    """try to return FUNC with ARGS, pass on ERRORS"""
     try:
-        ret = urllib.request.urlopen(url)
+        return func(*args)
+    except errors:
+        pass
+
+
+def retry(func, tries, *args):
+    """
+    Retry functions with potential connection errors.
+
+    *args are passed as variables to func.
+    """
+    _try = 1
+    while _try <= tries:
+        try:
+            answer = func(*args)
+            return answer
+        except (urllib.request.URLError, timeout):
+            time.sleep(1)
+            _try += 1
+
+
+def check_url(url, max_tries=1, time_out=15):
+    """Check if URL works. Returns bool"""
+
+    def _check_url(_url=url, _time_out=time_out):
+        ret = urllib.request.urlopen(_url, timeout=_time_out)
         # check return code for http(s) urls
-        if url.startswith("ftp") or ret.getcode() == 200:
+        if _url.startswith("ftp") or ret.getcode() == 200:
             return True
-    except urllib.request.URLError:
-        return False
+
+    return retry(_check_url, max_tries, url, time_out)
 
 
 def read_url(url):
@@ -501,7 +530,6 @@ def sanitize_annotation(genome):
         "\nGenome and annotation do not have matching sequence names! "
         "Creating matching annotation files...\n"
     )
-    # bgzip, genome_file = bgunzip_and_name(genome)
     genome_file, bgzip = gunzip_and_name(genome.filename)
 
     # try to find the (gtf) contig position in genome header
@@ -542,10 +570,9 @@ def sanitize_annotation(genome):
         for src, dst, gzip_file in zip(
             [new_gtf_file, new_bed_file],
             [gtf_file, bed_file],
-            [genome.annotation_gtf_file, genome.annotation_bed_file],
+            [gzip_file, genome.annotation_bed_file.endswith(".gz")],
         ):
             os.replace(src, dst)
-            gzip_file = True if gzip_file.endswith(".gz") else False
             gzip_and_name(dst, gzip_file)
 
     metadata, lines = read_readme(readme)
